
module eth_100G_controller_wrapper #(
    parameter MAX_CLUSTER_SIZE = 0,
    parameter FRAMEQ_DATA_LEN = 0,
    parameter FRAMEQ_META_LEN = 0
) (
    input clk,
    input rstn,
    output [47:0] mac_addr,
    output [31:0] ip_addr,
    output addr_valid,
    input si_tvalid,
    output si_tready,
    input [511:0] si_tdata,
    input [63:0] si_tkeep,
    input [7:0] si_tdest,
    input si_tlast,
    output eth_tx_tvalid,
    input eth_tx_tready,
    output [511:0] eth_tx_tdata,
    output [63:0] eth_tx_tkeep,
    output [0:0] eth_tx_tuser,
    output eth_tx_tlast,
    input s_axi_awvalid,
    output s_axi_awready,
    input [13:0] s_axi_awaddr,
    input [2:0] s_axi_awprot,
    input s_axi_wvalid,
    output s_axi_wready,
    input [31:0] s_axi_wdata,
    input [3:0] s_axi_wstrb,
    output s_axi_bvalid,
    input s_axi_bready,
    output [1:0] s_axi_bresp,
    input s_axi_arvalid,
    output s_axi_arready,
    input [13:0] s_axi_araddr,
    input [2:0] s_axi_arprot,
    output s_axi_rvalid,
    input s_axi_rready,
    output [31:0] s_axi_rdata,
    output [1:0] s_axi_rresp,
    input stat_rx_aligned,
    input stat_rx_aligned_err,
    input [2:0] stat_rx_bad_code,
    input stat_rx_bip_err_0,
    input stat_rx_bip_err_1,
    input stat_rx_bip_err_2,
    input stat_rx_bip_err_3,
    input stat_rx_bip_err_4,
    input stat_rx_bip_err_5,
    input stat_rx_bip_err_6,
    input stat_rx_bip_err_7,
    input stat_rx_bip_err_8,
    input stat_rx_bip_err_9,
    input stat_rx_bip_err_10,
    input stat_rx_bip_err_11,
    input stat_rx_bip_err_12,
    input stat_rx_bip_err_13,
    input stat_rx_bip_err_14,
    input stat_rx_bip_err_15,
    input stat_rx_bip_err_16,
    input stat_rx_bip_err_17,
    input stat_rx_bip_err_18,
    input stat_rx_bip_err_19,
    input stat_rx_hi_ber,
    input stat_rx_internal_local_fault,
    input stat_rx_misaligned,
    input [19:0] stat_rx_synced_err,
    input [3:0] gt_powergoodout,
    input usr_rx_reset,
    input usr_tx_reset,
    output ctl_tx_enable,
    output ctl_tx_send_rfi,
    output ctl_rx_enable
);
    eth_100G_controller #(
        .MAX_CLUSTER_SIZE(MAX_CLUSTER_SIZE),
        .FRAMEQ_DATA_LEN(FRAMEQ_DATA_LEN),
        .FRAMEQ_META_LEN(FRAMEQ_META_LEN)
    ) eth_100G_controller_I (
        .clk(clk),
        .rstn(rstn),
        .mac_addr(mac_addr),
        .ip_addr(ip_addr),
        .addr_valid(addr_valid),
        .si_tvalid(si_tvalid),
        .si_tready(si_tready),
        .si_tdata(si_tdata),
        .si_tkeep(si_tkeep),
        .si_tdest(si_tdest),
        .si_tlast(si_tlast),
        .eth_tx_tvalid(eth_tx_tvalid),
        .eth_tx_tready(eth_tx_tready),
        .eth_tx_tdata(eth_tx_tdata),
        .eth_tx_tkeep(eth_tx_tkeep),
        .eth_tx_tuser(eth_tx_tuser),
        .eth_tx_tlast(eth_tx_tlast),
        .s_axi_awvalid(s_axi_awvalid),
        .s_axi_awready(s_axi_awready),
        .s_axi_awaddr(s_axi_awaddr),
        .s_axi_awprot(s_axi_awprot),
        .s_axi_wvalid(s_axi_wvalid),
        .s_axi_wready(s_axi_wready),
        .s_axi_wdata(s_axi_wdata),
        .s_axi_wstrb(s_axi_wstrb),
        .s_axi_bvalid(s_axi_bvalid),
        .s_axi_bready(s_axi_bready),
        .s_axi_bresp(s_axi_bresp),
        .s_axi_arvalid(s_axi_arvalid),
        .s_axi_arready(s_axi_arready),
        .s_axi_araddr(s_axi_araddr),
        .s_axi_arprot(s_axi_arprot),
        .s_axi_rvalid(s_axi_rvalid),
        .s_axi_rready(s_axi_rready),
        .s_axi_rdata(s_axi_rdata),
        .s_axi_rresp(s_axi_rresp),
        .stat_rx_aligned(stat_rx_aligned),
        .stat_rx_aligned_err(stat_rx_aligned_err),
        .stat_rx_bad_code(stat_rx_bad_code),
        .stat_rx_bip_err_0(stat_rx_bip_err_0),
        .stat_rx_bip_err_1(stat_rx_bip_err_1),
        .stat_rx_bip_err_2(stat_rx_bip_err_2),
        .stat_rx_bip_err_3(stat_rx_bip_err_3),
        .stat_rx_bip_err_4(stat_rx_bip_err_4),
        .stat_rx_bip_err_5(stat_rx_bip_err_5),
        .stat_rx_bip_err_6(stat_rx_bip_err_6),
        .stat_rx_bip_err_7(stat_rx_bip_err_7),
        .stat_rx_bip_err_8(stat_rx_bip_err_8),
        .stat_rx_bip_err_9(stat_rx_bip_err_9),
        .stat_rx_bip_err_10(stat_rx_bip_err_10),
        .stat_rx_bip_err_11(stat_rx_bip_err_11),
        .stat_rx_bip_err_12(stat_rx_bip_err_12),
        .stat_rx_bip_err_13(stat_rx_bip_err_13),
        .stat_rx_bip_err_14(stat_rx_bip_err_14),
        .stat_rx_bip_err_15(stat_rx_bip_err_15),
        .stat_rx_bip_err_16(stat_rx_bip_err_16),
        .stat_rx_bip_err_17(stat_rx_bip_err_17),
        .stat_rx_bip_err_18(stat_rx_bip_err_18),
        .stat_rx_bip_err_19(stat_rx_bip_err_19),
        .stat_rx_hi_ber(stat_rx_hi_ber),
        .stat_rx_internal_local_fault(stat_rx_internal_local_fault),
        .stat_rx_misaligned(stat_rx_misaligned),
        .stat_rx_synced_err(stat_rx_synced_err),
        .gt_powergoodout(gt_powergoodout),
        .usr_rx_reset(usr_rx_reset),
        .usr_tx_reset(usr_tx_reset),
        .ctl_tx_enable(ctl_tx_enable),
        .ctl_tx_send_rfi(ctl_tx_send_rfi),
        .ctl_rx_enable(ctl_rx_enable)
    );
endmodule
