
module message_receiver_wrapper #(
    parameter MAX_DEVICES = 96,
    parameter STORE_PTID = 1,
    parameter AXI_ADDR_WIDTH = 34,
    parameter AXI_DATA_WIDTH = 512,
    parameter MSG_DATA_SIZE = 8960,
    parameter MBUFFER_ADDR_WIDTH = 30,
    parameter [AXI_ADDR_WIDTH-MBUFFER_ADDR_WIDTH-1:0] MBUFFER_ADDR_PREFIX = 4'b1111,
    parameter MAX_OUTSTANDING_WRITES = 8
) (
    input clk,
    input rstn,
    input [7:0] cluster_size,
    input msg_in_tvalid,
    output msg_in_tready,
    input [AXI_DATA_WIDTH-1:0] msg_in_tdata,
    input [AXI_DATA_WIDTH/8-1:0] msg_in_tkeep,
    input msg_in_tlast,
    output memcpy_arvalid,
    input memcpy_arready,
    output [AXI_ADDR_WIDTH-1:0] memcpy_araddr,
    output [7:0] memcpy_arlen,
    output [2:0] memcpy_arsize,
    output [1:0] memcpy_arburst,
    output [3:0] memcpy_arcache,
    input memcpy_rvalid,
    output memcpy_rready,
    input [AXI_DATA_WIDTH-1:0] memcpy_rdata,
    input memcpy_rlast,
    input [1:0] memcpy_rresp,
    output memcpy_awvalid,
    input memcpy_awready,
    output [AXI_ADDR_WIDTH-1:0] memcpy_awaddr,
    output [7:0] memcpy_awlen,
    output [2:0] memcpy_awsize,
    output [1:0] memcpy_awburst,
    output [3:0] memcpy_awcache,
    output memcpy_wvalid,
    input memcpy_wready,
    output [AXI_DATA_WIDTH-1:0] memcpy_wdata,
    output [AXI_DATA_WIDTH/8-1:0] memcpy_wstrb,
    output memcpy_wlast,
    input memcpy_bvalid,
    output memcpy_bready,
    input [1:0] memcpy_bresp,
    output bufwr_awvalid,
    input bufwr_awready,
    output [AXI_ADDR_WIDTH-1:0] bufwr_awaddr,
    output [7:0] bufwr_awlen,
    output [2:0] bufwr_awsize,
    output [1:0] bufwr_awburst,
    output [3:0] bufwr_awcache,
    output bufwr_wvalid,
    input bufwr_wready,
    output [AXI_DATA_WIDTH-1:0] bufwr_wdata,
    output [AXI_DATA_WIDTH/8-1:0] bufwr_wstrb,
    output bufwr_wlast,
    input bufwr_bvalid,
    output bufwr_bready,
    input [1:0] bufwr_bresp,
    input siCmd_tvalid,
    output siCmd_tready,
    input [63:0] siCmd_tdata,
    output soCmd_tvalid,
    input soCmd_tready,
    output [63:0] soCmd_tdata,
    output soCmd_tlast,
    input cntrl_arvalid,
    output cntrl_arready,
    input [2:0] cntrl_arprot,
    input [13:0] cntrl_araddr,
    output cntrl_rvalid,
    input cntrl_rready,
    output [31:0] cntrl_rdata,
    output [1:0] cntrl_rresp
);
    message_receiver #(
        .MAX_DEVICES(MAX_DEVICES),
        .STORE_PTID(STORE_PTID),
        .AXI_ADDR_WIDTH(AXI_ADDR_WIDTH),
        .AXI_DATA_WIDTH(AXI_DATA_WIDTH),
        .MSG_DATA_SIZE(MSG_DATA_SIZE),
        .MBUFFER_ADDR_WIDTH(MBUFFER_ADDR_WIDTH),
        .MBUFFER_ADDR_PREFIX(MBUFFER_ADDR_PREFIX),
        .MAX_OUTSTANDING_WRITES(MAX_OUTSTANDING_WRITES)
    ) message_receiver_I (
        .clk(clk),
        .rstn(rstn),
        .cluster_size(cluster_size),
        .msg_in_tvalid(msg_in_tvalid),
        .msg_in_tready(msg_in_tready),
        .msg_in_tdata(msg_in_tdata),
        .msg_in_tkeep(msg_in_tkeep),
        .msg_in_tlast(msg_in_tlast),
        .memcpy_arvalid(memcpy_arvalid),
        .memcpy_arready(memcpy_arready),
        .memcpy_araddr(memcpy_araddr),
        .memcpy_arlen(memcpy_arlen),
        .memcpy_arsize(memcpy_arsize),
        .memcpy_arburst(memcpy_arburst),
        .memcpy_arcache(memcpy_arcache),
        .memcpy_rvalid(memcpy_rvalid),
        .memcpy_rready(memcpy_rready),
        .memcpy_rdata(memcpy_rdata),
        .memcpy_rlast(memcpy_rlast),
        .memcpy_rresp(memcpy_rresp),
        .memcpy_awvalid(memcpy_awvalid),
        .memcpy_awready(memcpy_awready),
        .memcpy_awaddr(memcpy_awaddr),
        .memcpy_awlen(memcpy_awlen),
        .memcpy_awsize(memcpy_awsize),
        .memcpy_awburst(memcpy_awburst),
        .memcpy_awcache(memcpy_awcache),
        .memcpy_wvalid(memcpy_wvalid),
        .memcpy_wready(memcpy_wready),
        .memcpy_wdata(memcpy_wdata),
        .memcpy_wstrb(memcpy_wstrb),
        .memcpy_wlast(memcpy_wlast),
        .memcpy_bvalid(memcpy_bvalid),
        .memcpy_bready(memcpy_bready),
        .memcpy_bresp(memcpy_bresp),
        .bufwr_awvalid(bufwr_awvalid),
        .bufwr_awready(bufwr_awready),
        .bufwr_awaddr(bufwr_awaddr),
        .bufwr_awlen(bufwr_awlen),
        .bufwr_awsize(bufwr_awsize),
        .bufwr_awburst(bufwr_awburst),
        .bufwr_awcache(bufwr_awcache),
        .bufwr_wvalid(bufwr_wvalid),
        .bufwr_wready(bufwr_wready),
        .bufwr_wdata(bufwr_wdata),
        .bufwr_wstrb(bufwr_wstrb),
        .bufwr_wlast(bufwr_wlast),
        .bufwr_bvalid(bufwr_bvalid),
        .bufwr_bready(bufwr_bready),
        .bufwr_bresp(bufwr_bresp),
        .siCmd_tvalid(siCmd_tvalid),
        .siCmd_tready(siCmd_tready),
        .siCmd_tdata(siCmd_tdata),
        .soCmd_tvalid(soCmd_tvalid),
        .soCmd_tready(soCmd_tready),
        .soCmd_tdata(soCmd_tdata),
        .soCmd_tlast(soCmd_tlast),
        .cntrl_arvalid(cntrl_arvalid),
        .cntrl_arready(cntrl_arready),
        .cntrl_arprot(cntrl_arprot),
        .cntrl_araddr(cntrl_araddr),
        .cntrl_rvalid(cntrl_rvalid),
        .cntrl_rready(cntrl_rready),
        .cntrl_rdata(cntrl_rdata),
        .cntrl_rresp(cntrl_rresp)
    );
endmodule
